;============================ ROTATOR PROCEDURE =====================================================
; Clock is set to 112.3uS
; counter counts from 255 and down to measure the rotator speed.
;
; Value from 246-255 is considered as bounces and should be set to one increament.
; Values from 0 to 181 is considered as slow speed and also set to one increament.
; Values from 182 to 245  (63 values) is the sped.
; Speed is divided by 16 to give 0 - 3 as value.
; Total rotator speed will be 0 to 4.
;
; Bit 7 indicate that rotation has occured.  (1 = rotation has happened, new value exist)
;
; Bit 6 indicate clockwise or counter-clockwise. (1 = negative rotation counter-clock wise)
;
;            A  +  B
;            |  |  |
;            -------
;           |       |
;           |   O   |   Top view
;           |       |
;            -------
;
;TESTING Counter -clock wise ---------------------------------------------
ROTARY:    BTFSC ROTPA,INP_A  ;Do next instr if INP_A is 1:a  
           GOTO CC_WISE       ;
           BTFSS ROTPB,INP_B  ;Do next instr if INP_B is 0:a
           RETURN             ;Rotary is not activated, do nothing more
; B input was high
           BTFSS ROTPA,INP_A  ;Do next instr if INP_A is 0:a  
           GOTO WAIT_F_A      ;Wait for a to go high
;A is high not good, so go back to start
WAIT_LOW:  BTFSC ROTPA,INP_A  ;Do next instr if INP_A is 1:a  
           GOTO WAIT_LOW      ;Wait until A is low again
;Pause 500uS for buns then go back
           CALL T500US        ;500 uS
;A is high not good, so go back to start
           RETURN             ;Start all over
;
;Measre the time A is high
WAIT_F_A:  MOVLW 0FFh         ;Set speed to max
           MOVWF ROTARY_DAT   ;
;Wait for A to go high
WAIT_HH:   BTFSC ROTPA,INP_A  ;Do next instr if INP_A is 1:a  
           GOTO AH_OK         ;
;Test B which must be 1 else something is wrong and if so jump out
           BTFSC ROTPB,INP_B  ;Do next instr if INP_B is 1:a
           GOTO WAIT_HH       ;Input was low go back and test again
           CLRF ROTARY_DAT    ;
           RETURN             ;Rotary is not activated, do nothing more

;Pulse was high
;Pause 500uS
AH_OK:     CALL T500US        ;Pause 500uS

;Check if A is still high
STILL_TIME:BTFSS ROTPA,INP_A  ;Do next instr if INP_A is 0:a  
           GOTO A_GOT_LOW     ;A Went low so go out of loop
;A is still high check if max time has been reached
;Pause 100uS
           CALL T112US        ;Pause 112uS
;Pause 100uS
;           CALL T100US        ;Pause 100uS
           DECFSZ ROTARY_DAT,F;Do next instruction if register not reached Zero
           GOTO STILL_TIME    ; 
;Max time reached so go out of loop
;Check if A is low
A_WAIT_2:  BTFSC ROTPA,INP_A  ;Do next instr if INP_A is 1:a  
           GOTO A_WAIT_2      ;
;Pause 500uS for buns then go back
           CALL T500US        ;500 uS
;Time out occured so set the speed to lowest!
           MOVLW b'11000000'  ;Rotation occured, negative rotation, lowest speed
           MOVWF ROTARY_DAT   ;
           RETURN
;Rotation occured OK.
A_GOT_LOW: CALL ROT_CALC      ;Test the limit of the pulse time
           BSF ROTARY_DAT,6   ;Set bit 6 to indicate negative rotation
           RETURN
;TESTING Clock wise ----------------------------------------------------------------
;
; A input was high
CC_WISE:   BTFSS ROTPB,INP_B  ;Do next instr if INP_B is 0:a  
           GOTO WAIT_F_B      ;Wait for B to go high
;B is high not good, so go back to start
WAIT_LOW_B:BTFSC ROTPB,INP_B  ;Do next instr if INP_B is 1:a  
           GOTO WAIT_LOW_B    ;Wait until B is low again
;Pause 500uS for buns then go back
           CALL T500US        ;500 uS
;A is high not good, so go back to start
           RETURN             ;Start all over
;
;Measure the time B is high
WAIT_F_B:  MOVLW 0FFh         ;Set speed to max
           MOVWF ROTARY_DAT   ;
;Wait for B to go high
WAIT_HHB:  BTFSC ROTPB,INP_B  ;Do next instr if INP_B is 1:a  
           GOTO BH_OK         ;
;Test A which must be 1 else something is wrong and if so jump out
           BTFSC ROTPA,INP_A  ;Do next instr if INP_A is 1:a
           GOTO WAIT_HHB      ;Input was low go back and test again
           CLRF ROTARY_DAT    ;
           RETURN             ;Rotary is not activated, do nothing more
;Pulse was high
;Pause 500uS
BH_OK:     CALL T500US        ;Pause 500uS

;Check if B is still high
STILL_T_B: BTFSS ROTPB,INP_B  ;Do next instr if INP_B is 0:a  
           GOTO B_GOT_LOW     ;B Went low so go out of loop
;A is still high check if max time has been reached
;Pause 112uS
           CALL T112US        ;Pause 112uS
;
           DECFSZ ROTARY_DAT,F;Do next instruction if register not reached Zero
           GOTO STILL_T_B     ; 
;Max time reached so go out of loop
;Check if B is low
B_WAIT_2:  BTFSC ROTPB,INP_B  ;Do next instr if INP_B is 1:a  
           GOTO B_WAIT_2      ;
;Pause 500uS for buns then go back
           CALL T500US        ;500 uS
;Time out occured so set the speed to lowest!
           MOVLW b'10000000'  ;Rotation occured, positive rotation, lowest speed
           MOVWF ROTARY_DAT   ;
           RETURN
;Rotation occured OK.
B_GOT_LOW: CALL ROT_CALC      ;Test the limit of the pulse time
           RETURN

;---------------------------------------------------------------------
;This procedure check the limits of the pulse time
;Check if value is between 246 - 255 which means bad value (= 1 step)
;Value from 182 to 245 is OK. Values above and below is set to one step.
;
; --181---182---------------245---246-------255
;  low  |        ok             |   not OK
; 
;                
;
ROT_CALC:  MOVF ROTARY_DAT,W  ;
           MOVWF TAL1LO       ;
           CLRF TAL1HI        ;
           MOVLW 0f6h         ;246
           MOVWF TAL2LO       ;
           CLRF TAL2HI        ;
           CALL XSUB          ;ROTARY_DAT-246 = ?
;test if result is negative which means that values is less than 246
           BTFSC TAL2HI,7     ;Perform next insts if bit = 1  
           GOTO LESS_246      ;ROTARY_DAT < 246 
;Rotary value is larger than 245
           MOVLW b'10000000'  ;lowest speed
           MOVWF ROTARY_DAT   ;
           RETURN
;
;Check if value is larger than 181
LESS_246:  MOVF ROTARY_DAT,W  ;
           MOVWF TAL1LO       ;
           CLRF TAL1HI        ;
           MOVLW 0b6h         ;182
           MOVWF TAL2LO       ;
           CLRF TAL2HI        ;
           CALL XSUB          ;ROTARY_DAT-182 = ?
;test if result is negative which means that values is less than 182
           BTFSC TAL2HI,7     ;Perform next insts if bit = 1  
           GOTO LESS_182      ;ROTARY_DAT < 182 
;Rotary value is larger than 181 so subtract and divide by 16
;
           MOVF ROTARY_DAT,W  ;
           MOVWF TAL1LO       ;
           CLRF TAL1HI        ;
           MOVLW 0b6h         ;182
           MOVWF TAL2LO       ;
           CLRF TAL2HI        ;
           CALL XSUB          ;ROTARY_DAT-182 = value is from 0 to 63
;Save result
           MOVF TAL2LO,W      ;
           MOVWF ROTARY_DAT   ;
;Rotary contain value from 0 to 63, now divide it by 16
           RRF ROTARY_DAT,F    ;divide by 16
           RRF ROTARY_DAT,F    ;
           RRF ROTARY_DAT,F    ;
           RRF ROTARY_DAT,F    ;
           INCF ROTARY_DAT,F   ;Inc one step so speed should go from 1 to 4 
           MOVLW b'00000111'   ;Mask away junk
           ANDWF ROTARY_DAT,F  ;
           BSF ROTARY_DAT,7    ;Set bit 7 to indicate action has happened
           RETURN              ;
;
;Speed is slow so set value 10000000 
LESS_182:  MOVLW b'10000000'  ;lowest speed
           MOVWF ROTARY_DAT   ;
           RETURN

;---------------------------------------------------------------------
;Timeloop Iteration_Number * 16 + 26
;
;T100US 06 hex = 123 cycles med 4.9152 MHz crystal (100 uS)
BUNS_TIM:  EQU	06h           ;Bunsing time constant
;
T100US:    CLRF T_LOOP_3      ;
           CLRF T_LOOP_2      ;
           MOVLW BUNS_TIM     ;
           MOVWF T_LOOP_1     ;
           CALL T_STRT        ;
           RETURN             ;

;T1129US 07 hex = 138 cycles med 4.9152 MHz crystal (112.3 uS)
;
T112US:    CLRF T_LOOP_3      ;
           CLRF T_LOOP_2      ;
           MOVLW 07h          ;
           MOVWF T_LOOP_1     ;
           CALL T_STRT        ;
           RETURN             ;
;
T500US:    CALL T100US        ;
           CALL T100US        ;
           CALL T100US        ;
           CALL T100US        ;
           CALL T100US        ;
           RETURN             ;
;
;===========================================================================
; Procedure to subtrackt, for testing the rotary speed
;
;        Proceduren ADD adderar talen Tal1 med Tal2  Tal2 <= (Tal1+tal2)
;        Proceduren SUB subtraheraren Tal1 med Tal2  Tal2 <= (Tal1-tal2)
;
NEG_TAL2:  COMF TAL2LO,F      ;
           INCF TAL2LO,F      ;
           BTFSC STATUS,Z     ;Om Z=1 utfr nsta instr. (Z=1 vid nollres.)
           DECF TAL2HI,F      ;
           COMF TAL2HI,F      ;
           RETURN             ;
;
;Subtrahera -------------------------------------------------------------
;
XSUB:      CALL NEG_TAL2      ;
;
;Addera -----------------------------------------------------------------
;
XADD:      MOVF TAL1LO,W      ;Ladda Tal1 lgdel
           ADDWF TAL2LO,F     ;
           BTFSC STATUS,C     ;Om C=1 utfr nsta instr. (C=1 vid carry)
           INCF TAL2HI,F      ;
           MOVF TAL1HI,W      ;
           ADDWF TAL2HI,F     ;
           RETURN
;===========================================================================